@@ -420,12 +420,13 @@ write_module(ModName, ModName, AST, FunDeps, ErlDirName,
     Registrar = make_registrar(ModName),
     Decoders = make_decoders(TaggedElems, ModName, ModName),
     Encoders = make_encoders(TaggedElems, ModName),
-    Printer = make_printer(TaggedElems, PredefRecords, ModName),
+    Printer = make_printer(TaggedElems, PredefRecords, ModName, ModName),
+    GettersSetters = make_getters_setters(TaggedElems, PredefRecords, ModName),
     Resolver = make_resolver(TaggedElems, ModName),
     AuxFuns = make_aux_funs(),
     LocalFunForms = make_local_funs(FunDeps, TaggedElems, ModName),
     NewAST = TopDecoders ++ TopEncoders ++ Decoders ++ Encoders ++
-	Registrar ++ AuxFuns ++ LocalFunForms ++ Printer ++ Resolver ++ AST,
+	GettersSetters ++ Registrar ++ AuxFuns ++ LocalFunForms ++ Printer ++ Resolver ++ AST,
     Compile = erl_syntax:attribute(?AST(compile), [?AST(export_all)]),
     Hdr = header(SpecFile),
     ResultAST = erl_syntax:form_list([Hdr, Module, Compile|NewAST]),
@@ -442,10 +443,11 @@ write_module(ParentMod, ModName, AST, FunDeps, ErlDirName,
                [erl_syntax:atom(ModName)]),
     Decoders = make_decoders(TaggedElems, ParentMod, ModName),
     Encoders = make_encoders(TaggedElems, ModName),
-    Printer = make_printer(TaggedElems, PredefRecords, ModName),
+    Printer = make_printer(TaggedElems, PredefRecords, ModName, ParentMod),
+    GettersSetters = make_getters_setters(TaggedElems, PredefRecords, ModName),
     Compile = erl_syntax:attribute(?AST(compile), [?AST(export_all)]),
     LocalFunForms = make_local_funs(FunDeps, TaggedElems, ModName),
-    NewAST = Decoders ++ Encoders ++ Printer ++ LocalFunForms ++ AST,
+    NewAST = Decoders ++ Encoders ++ GettersSetters ++ Printer ++ LocalFunForms ++ AST,
     ResultAST = erl_syntax:form_list([Hdr, Module, Compile|NewAST]),
     file:write_file(
       filename:join([ErlDirName, ModNameErl]),
@@ -620,7 +622,7 @@ make_typespecs(_ModName, {_Tags, _TypesDict, RecDict}, Opts) ->
     case proplists:get_value(add_type_specs, Opts) of
 	TypeName when is_atom(TypeName), TypeName /= undefined ->
 	    case [[atom_to_string(R), "()"]
-		  || {record, R} <- dict:fetch_keys(RecDict)] of
+		  || {record, R} <- dict_keys(RecDict)] of
 		[] ->
 		    [];
 		Records ->
@@ -828,16 +830,6 @@ make_top_encoders(_TaggedSpecs, _ModName) ->
 		none,
 		[?AST(Mod = get_mod(El)),
 		 ?AST(Mod:do_encode(El, TopXMLNS))]),
-    PPCase = erl_syntax:case_expr(
-	       ?AST(get_mod(Term)),
-	       [erl_syntax:clause(
-		  [?AST(undefined)],
-		  none,
-		  [?AST(io_lib_pretty:print(Term, fun (_, _) -> no end))]),
-		erl_syntax:clause(
-		  [?AST(Mod)],
-		  none,
-		  [?AST(io_lib_pretty:print(Term, fun Mod:pp/2))])]),
     [make_function(encode, [?AST(El)], [?AST(encode(El, <<>>))]),
      erl_syntax:function(?AST(encode), [Clause1, Clause2]),
      make_function(get_name, [?AST(El)], GetNameCase),
@@ -845,7 +837,12 @@ make_top_encoders(_TaggedSpecs, _ModName) ->
      make_function(is_known_tag,
 		   [?AST({xmlel, Name, Attrs, _}), ?AST(TopXMLNS)],
 		   KnownTagCase),
-     make_function(pp, [?AST(Term)], [PPCase])].
+     make_function(get_els, [?AST(Term)],
+		   [?AST(Mod = get_mod(Term)),
+		    ?AST(Mod:get_els(Term))]),
+     make_function(set_els, [?AST(Term), ?AST(Els)],
+		   [?AST(Mod = get_mod(Term)),
+		    ?AST(Mod:set_els(Term, Els))])].
 
 make_encoders(TaggedSpecs, ModName) ->
     {RecNames, ResNames} =
@@ -954,11 +951,28 @@ make_encoders(TaggedSpecs, ModName) ->
        true -> []
     end.
 
-make_printer(TaggedSpecs, PredefRecords, ModName) ->
-    PassClause1 = erl_syntax:clause(
-                   [?AST(_), ?AST(_)],
-                   none,
-                   [?AST(no)]),
+make_printer(TaggedSpecs, PredefRecords, ModName, ParentMod) ->
+    PassClause1 =
+	if ModName == ParentMod ->
+		[erl_syntax:clause(
+		   [?AST(xmlel), ?AST(3)],
+		   none,
+		   [?AST([name, attrs, children])]),
+		 erl_syntax:clause(
+		   [?AST(Name), ?AST(Arity)],
+		   none,
+		   [erl_syntax:case_expr(
+		      ?AST(get_mod(
+			     erlang:make_tuple(Arity+1, undefined, [{1, Name}]))),
+		      [erl_syntax:clause(
+			 [?AST(undefined)], none, [?AST(no)]),
+		       erl_syntax:clause(
+			 [?AST(Mod)],
+			 none,
+			 [?AST(Mod:pp(Name, Arity))])])])];
+	   true ->
+		[erl_syntax:clause([?AST(_), ?AST(_)], none, [?AST(no)])]
+	end,
     %% Exclude tags with duplicated results
     RecNames = lists:foldl(
                  fun({Tag, #elem{result = Result, module = Mod}}, Acc)
@@ -999,16 +1013,69 @@ make_printer(TaggedSpecs, PredefRecords, ModName) ->
                           {Acc1, Acc2}
                   end
           end, {[], []}, TaggedSpecs),
-    if Clauses1 /= [] ->
-	    [erl_syntax:function(?AST(pp), Clauses1 ++ [PassClause1])];
-       true ->
-	    []
-    end ++ [make_function(records, [],
-			  [erl_syntax:list(
-			     lists:map(
-			       fun({RecName, RecSize}) ->
-				       ?AST({'?a(RecName)', '?a(RecSize)'})
-			       end, Records))])].
+    [erl_syntax:function(?AST(pp), Clauses1 ++ PassClause1),
+     make_function(records, [],
+		   [erl_syntax:list(
+		      lists:map(
+			fun({RecName, RecSize}) ->
+				?AST({'?a(RecName)', '?a(RecSize)'})
+			end, Records))])].
+
+make_getters_setters(TaggedSpecs, PredefRecords, ModName) ->
+    RecNames = lists:foldl(
+                 fun({Tag, #elem{result = Result, module = Mod}}, Acc)
+		       when Mod == ModName ->
+                         try
+                             [H|_]= tuple_to_list(Result),
+                             true = is_atom(H),
+                             false = is_label(H),
+                             dict:append(H, Tag, Acc)
+                         catch _:_ ->
+                                 Acc
+                         end;
+		    (_, Acc) ->
+			 Acc
+                 end, dict:new(), TaggedSpecs),
+    {Getters, Setters} =
+        lists:foldl(
+          fun({Tag, #elem{result = Result}}, {Acc1, Acc2}) ->
+                  try
+                      [H|T]= tuple_to_list(Result),
+                      true = is_atom(H),
+                      false = is_label(H),
+                      [Tag|_] = dict:fetch(H, RecNames),
+		      Fields = case dict:find(H, PredefRecords) of
+				   {ok, Fs} ->
+				       [FName || {FName, _} <- Fs];
+				   error ->
+				       [label_to_record_field(F) || F <- T]
+			       end,
+		      case lists:member(sub_els, Fields) of
+			  false ->
+			      {Acc1, Acc2};
+			  true ->
+			      {[erl_syntax:clause(
+				  [record_fields_to_vars(H, Fields)],
+				  none,
+				  [?AST(_sub_els)])|Acc1],
+			       [erl_syntax:clause(
+				  [record_fields_to_vars(H, Fields, [{sub_els, ?AST(_)}]),
+				   ?AST(_sub_els)],
+				  none,
+				  [record_fields_to_vars(
+				     H, Fields, [{sub_els, ?AST(_sub_els)}])])|Acc2]}
+		      end
+		  catch _:_ ->
+			 {Acc1, Acc2}
+                  end
+          end, {[], []}, TaggedSpecs),
+    case {Getters, Setters} of
+	{[], []} ->
+	    [];
+	_ ->
+	    [erl_syntax:function(?AST(get_els), Getters),
+	     erl_syntax:function(?AST(set_els), Setters)]
+    end.
 
 elem_to_AST(#elem{name = Name, xmlns = XMLNS, cdata = CData,
                   result = Result, attrs = Attrs, refs = _Refs} = Elem,
@@ -1981,6 +2048,22 @@ label_to_record_field(Label) ->
             list_to_atom(T)
     end.
 
+record_fields_to_vars(Name, Fields) ->
+    record_fields_to_vars(Name, Fields, []).
+
+record_fields_to_vars(Name, Fields, Replace) ->
+    FList = lists:map(
+	      fun(Field) ->
+		      case lists:keyfind(Field, 1, Replace) of
+			  {_, AbstractVal} ->
+			      AbstractVal;
+			  false ->
+			      erl_syntax:variable(
+				[$_|atom_to_list(Field)])
+		      end
+	      end, Fields),
+    erl_syntax:tuple([erl_syntax:atom(Name)|FList]).
+
 prepare_label(Label, Name) when is_atom(Name) ->
     prepare_label(Label, erlang:atom_to_binary(Name, utf8));
 prepare_label(Label, Name) ->
@@ -2162,7 +2245,7 @@ get_types(TaggedElems, FunSpecs, Opts) ->
                           Type = term_to_t(Result, LabelTypes),
                           dict:store(RefName, Type, Dict)
                   end, dict:new(), SortedTags),
-    RecDict = dict:from_list(
+    RecDict = dict_from_list(
                 lists:flatmap(
                   fun({Tag, _T}) ->
                           RefElem = get_elem_by_ref(Tag, TaggedElems),
@@ -2739,8 +2822,20 @@ t_from_form(Spec) ->
     T.
 
 t_remote(Mod, Type) ->
-    D = dict:from_list([{{opaque, Type, []},
+    D = dict_from_list([{{opaque, Type, []},
 			 {{Mod, 1, 2, []}, type}}]),
     [T] = erl_types:t_opaque_from_records(D),
     T.
 -endif.
+
+-ifdef(USE_MAPS).
+dict_from_list(L) ->
+    maps:from_list(L).
+dict_keys(D) ->
+    maps:keys(D).
+-else.
+dict_from_list(L) ->
+    dict:from_list(L).
+dict_keys(D) ->
+    dict:fetch_keys(D).
+-endif.
